﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Collections.ObjectModel;

namespace WCMexport
{
    public partial class WCMexportForm : Form
    {
        WCMexportLib wcmExportLib;
        TreeNode tnMasterPages;
        TreeNode tnPageLayouts;
        TreeNode tnStyleLibraryFiles;

        public WCMexportForm()
        {
            InitializeComponent();
        }

        private void btnGetContent_Click(object sender, EventArgs e)
        {
            this.Cursor = Cursors.WaitCursor;
            if (wcmExportLib != null)
            {
                wcmExportLib.Dispose();
                wcmExportLib = null;
            }
            try
            {
                wcmExportLib = new WCMexportLib(txtSiteUrl.Text);
                treeContent.BeginUpdate();
                treeContent.Nodes.Clear();
                tnMasterPages = AddContent("Master Pages", wcmExportLib.GetMasterPages());
                tnPageLayouts = AddContent("Page Layouts", wcmExportLib.GetPageLayouts());
                tnStyleLibraryFiles = AddDirectoryContent("Style Library Files", wcmExportLib.GetStyleLibraryFiles(), "Style Library");
                treeContent.EndUpdate();
                treeContent.Enabled = true;
                btnExport.Enabled = true;
            }
            catch (Exception ex)
            {
                treeContent.BeginUpdate();
                treeContent.Nodes.Clear();
                treeContent.Enabled = false;
                treeContent.EndUpdate();
                btnExport.Enabled = false;
                MessageBox.Show(ex.Message);
            }
            this.Cursor = Cursors.Default;
        }

        private TreeNode AddContent(string rootNodeName, ReadOnlyCollection<UrlInfo> urlInfoCollection)
        {
            TreeNode tnRoot = new TreeNode(rootNodeName);
            tnRoot.Checked = true;
            foreach (UrlInfo ui in urlInfoCollection)
            {
                TreeNode tn = new TreeNode(ui.Title);
                tn.Checked = true;
                tn.ToolTipText = ui.InfoString;
                tn.Tag = ui.Url;
                tnRoot.Nodes.Add(tn);
            }
            if (tnRoot.Nodes.Count > 0)
            {
                treeContent.Nodes.Add(tnRoot);
                return tnRoot;
            }
            return null;
        }

        private TreeNode AddDirectoryContent(string rootNodeName, ReadOnlyCollection<UrlInfo> urlInfoCollection, string rootDir)
        {
            TreeNode tnRoot = new TreeNode(rootNodeName);
            tnRoot.Checked = true;
            tnRoot.Tag = rootDir;
            List<TreeNode> tnList = new List<TreeNode>();
            tnList.Add(tnRoot);
            foreach (UrlInfo ui in urlInfoCollection)
            {
                TreeNode tn = new TreeNode(ui.Title);
                tn.Checked = true;
                tn.ToolTipText = ui.InfoString;
                tn.Tag = ui.Url;

                string[] urlParts = ui.Url.Substring(ui.Url.IndexOf(rootDir)).Split(new char[] { '/' });
                int i,j;
                for (i = 0; i < urlParts.Length-1; i++)
                {
                    if (tnList.Count<i+1 || (string)tnList[i].Tag != urlParts[i])
                        break;
                }
                while (i < tnList.Count)
                {
                    tnList.RemoveAt(i);
                }
                for (j = i; j < urlParts.Length-1; j++)
                {
                    TreeNode tnDir = new TreeNode(urlParts[j]);
                    tnDir.Checked = true;
                    tnDir.Tag = urlParts[j];
                    tnList[j - 1].Nodes.Add(tnDir);
                    tnList.Add(tnDir);
                }

                tnList[j-1].Nodes.Add(tn);
            }
            if (tnRoot.Nodes.Count > 0)
            {
                treeContent.Nodes.Add(tnRoot);
                return tnRoot;
            }
            return null;
        }

        private void CheckAllChildNodes(TreeNode treeNode, bool nodeChecked)
        {
            foreach (TreeNode node in treeNode.Nodes)
            {
                node.Checked = nodeChecked;
                if (node.Nodes.Count > 0)
                {
                    this.CheckAllChildNodes(node, nodeChecked);
                }
            }
        }

        private void node_AfterCheck(object sender, TreeViewEventArgs e)
        {
            // The code only executes if the user caused the checked state to change.
            if (e.Action != TreeViewAction.Unknown)
            {
                if (e.Node.Nodes.Count > 0)
                {
                    this.CheckAllChildNodes(e.Node, e.Node.Checked);
                }
            }
        }

        private void txtSiteUrl_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
            {
                btnGetContent_Click(null, null);
            }
        }

        private void btnExport_Click(object sender, EventArgs e)
        {
            if (saveFileDialog.ShowDialog() == DialogResult.OK)
            {
                this.Cursor = Cursors.WaitCursor;
                try
                {
                    using (WCMexportLib.Exporter exporter = wcmExportLib.StartExport(saveFileDialog.FileName))
                    {
                        exportChildNodes(tnMasterPages, exporter.AddMasterPage);
                        exportChildNodes(tnPageLayouts, exporter.AddPageLayout);
                        exportChildNodes(tnStyleLibraryFiles, exporter.AddStyleLibraryFile);
                    };
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
                this.Cursor = Cursors.Default;
            }
        }

        private void exportChildNodes(TreeNode tn, Action<string> exportFunction)
        {
            if (tn==null)
                return;
            if (tn.Nodes.Count == 0 && tn.Checked)
            {
                exportFunction((string)tn.Tag);
            }
            else
            {
                foreach (TreeNode child in tn.Nodes)
                {
                    exportChildNodes(child, exportFunction);
                }
            }
        }
    }
}
